import { CSSObject } from 'styled-components';

export type HiroColors =
  | 'primary'
  | 'secondary'
  | 'primaryText'
  | 'secondaryText'
  | 'red'
  | 'orange'
  | 'yellow'
  | 'green'
  | 'teal'
  | 'blue'
  | 'purple'
  | 'pink'
  | 'brown'
  | 'black'
  | 'grey'
  | 'white';

export type Greyscales = 'light' | 'medium' | 'dark' | 'darker' | 'darkest';

type RecursivePartial<T> = {
  // Stop recursion from exploding
  [P in keyof T]: T[P] extends CSSObject ? T[P] : RecursivePartial<T[P]>;
};

export interface ColorDefinitionValues {
  default: string;
  active: string;
  hover: string;
  light: string;
  verylight: string;
}

export type ColorDefinitions = Record<HiroColors, ColorDefinitionValues>;

export interface FontDefinitons {
  h1: CSSObject;
  h2: CSSObject;
  h3: CSSObject;
}

export type GreyscaleDefinitions = {
  [K in Greyscales]: string;
};

export interface SizeDefinitions {
  borderRadius: string | number;
  padding: string;
}

export interface HiroDefaultTheme {
  breakpoints: string[];
  colors: Record<HiroColors, string>;
  font: FontDefinitons;
  global: CSSObject;
  greyscale: GreyscaleDefinitions;
  sizes: SizeDefinitions;
}
export type HiroPartialTheme = RecursivePartial<HiroDefaultTheme>;

declare module 'styled-components' {
  export interface DefaultTheme {
    breakpoints: string[];
    colors: ColorDefinitions;
    font: FontDefinitons;
    global: CSSObject;
    greyscale: GreyscaleDefinitions;
    sizes: SizeDefinitions;
  }
}
