import { CSSObject, DefaultTheme } from 'styled-components';

export const flexbox = <
  T extends { size?: number | number[]; theme: DefaultTheme }
>({
  size,
  theme,
}: T) => {
  if (!size) {
    return;
  }

  const { breakpoints } = theme;

  const res: CSSObject = {
    flexBasis: '100%',
  };

  if (!Array.isArray(size)) {
    res['@media screen and (min-width: 64em)'] = {
      flexBasis: `${100 * size}%`,
    };

    return res;
  }

  for (let i = 0; i < size.length; i++) {
    if (breakpoints.length < i) {
      break;
    }

    const query = `@media screen and (min-width: ${breakpoints[i]})`;

    res[query] = {
      flexBasis: `${100 * size[i]}%`,
    };
  }

  return res;
};

const alignable = (isRow?: boolean) => <
  T extends {
    verticalAlign?: 'start' | 'center' | 'end';
    centered?: boolean;
    floated?: 'left' | 'right';
  }
>({
  verticalAlign,
  centered,
  floated,
}: T): CSSObject | null => {
  const s: CSSObject = {
    justifyContent: undefined,
    alignItems: undefined,
  };

  if (verticalAlign) {
    s[isRow ? 'alignItems' : 'justifyContent'] = verticalAlign;
  }

  if (floated) {
    s[!isRow ? 'alignItems' : 'justifyContent'] =
      floated === 'left' ? 'flex-start' : 'flex-end';

    s[floated === 'left' ? 'marginRight' : 'marginLeft'] = 'auto';
  }

  if (centered) {
    s[!isRow ? 'alignItems' : 'justifyContent'] = 'center';
  }

  if (isRow) {
    s.width = '100%';
  }

  return s;
};

export const alignRow = alignable(true);
export const alignColumn = alignable();
